@@ -237,8 +237,7 @@ function (Z_mat, fixed_effects, control)
     if (length(na_list) > 0) {
         teachyearcomb <- unique(cBind(Z_mat[-na_list, ]$year, 
             Z_mat[-na_list, ]$teacher))
-    }
-    else {
+    }else {
         teachyearcomb <- unique(cBind(Z_mat$year, Z_mat$teacher))
     }
     nteach_effects <- sum(nyear - as.numeric(teachyearcomb[, 
@@ -291,16 +290,20 @@ function (Z_mat, fixed_effects, control)
     n_eta <- nteach_effects
     n_ybeta <- dim(X_mat)[2]
     Z <- Matrix(RE_mat)
+    huge.flag<-prod(dim(Z))>1e8
     Y <- as.vector(Z_mat$y)
     X <- Matrix(X_mat)
+    if(!huge.flag){
     Z.dense <- as.matrix(Z)
     X.dense <- as.matrix(X)
+    }
     Z_mat.full$r <- 1 - Z_mat.full$mis
     pattern.student <- matrix(Z_mat.full$r, nstudent, nyear, 
         byrow = TRUE)
     Z_mat.full$pat <- rep(apply(pattern.student, 1, bin2dec), 
         each = nyear)
     Z_mat$pat <- Z_mat.full[Z_mat.full$r == 1, ]$pat
+    rm(Z_mat.full)
     pat <- list()
     pattern.count <- list()
     pattern.length <- list()
@@ -310,9 +313,14 @@ function (Z_mat, fixed_effects, control)
     pattern.key <- dec2bin(1:(2^nyear - 1))
     for (p in unique(Z_mat$pat)) {
         pat[[p]] <- which(Z_mat$pat == p)
-        X.p[[p]] <- X.dense[pat[[p]], , drop = FALSE]
-        Y.p[[p]] <- Y[pat[[p]]]
+        if(!huge.flag){
+        X.p[[p]] <- X.dense[pat[[p]], , drop = FALSE]     
         Z.p[[p]] <- Z.dense[pat[[p]], , drop = FALSE]
+        }else{
+        X.p[[p]] <- X[pat[[p]], , drop = FALSE]     
+        Z.p[[p]] <- Z[pat[[p]], , drop = FALSE]
+        }
+        Y.p[[p]] <- Y[pat[[p]]]
         pattern.count[[p]] <- length(Y.p[[p]])
         pattern.length[[p]] <- sum(pattern.key[p, ])
     }
@@ -499,8 +507,13 @@ function (Z_mat, fixed_effects, control)
                   pattern.length[[p]]), , drop = FALSE]
                 Y.t <- Y.p[[p]][(1 + (i - 1) * pattern.length[[p]]):(i * 
                   pattern.length[[p]])]
+                if(!huge.flag){
                 Z.t <- Z.p[[p]][(1 + (i - 1) * pattern.length[[p]]):(i * 
                   pattern.length[[p]]), , drop = FALSE]
+                  }else{
+                Z.t <- as.matrix(Z.p[[p]][(1 + (i - 1) * pattern.length[[p]]):(i * 
+                  pattern.length[[p]]), , drop = FALSE])                  
+                  }
                 temp.t <- Y.t - X.t %*% ybetas
                 quickm <- c(NULL)
                 for (a in 1:pattern.length[[p]]) {
@@ -587,8 +600,13 @@ if(hes.count==29) R.cc<-0
                   pattern.length[[p]]), , drop = FALSE]
                 Y.t <- Y.p[[p]][(1 + (i - 1) * pattern.length[[p]]):(i * 
                   pattern.length[[p]])]
+                if(!huge.flag){
                 Z.t <- Z.p[[p]][(1 + (i - 1) * pattern.length[[p]]):(i * 
                   pattern.length[[p]]), , drop = FALSE]
+                  }else{
+                Z.t <- as.matrix(Z.p[[p]][(1 + (i - 1) * pattern.length[[p]]):(i * 
+                  pattern.length[[p]]), , drop = FALSE])                  
+                  }
                 temp.t <- Y.t - X.t %*% ybetas
                 quickm <- c(NULL)
                 for (a in 1:pattern.length[[p]]) {
@@ -679,7 +697,7 @@ if(hes.count==29) R.cc<-0
     if (control$hessian == TRUE) {
         parameters <- round(cBind(c(ybetas, thetas), c(ybetas_stderror, 
             std_errors)), 4)
-        colnames(parameters) <- c("Estimate", "Etandard Error")
+        colnames(parameters) <- c("Estimate", "Standard Error")
         rownames(parameters) <- as.character(effect_la)
     }
     if (control$hessian == FALSE) {
@@ -710,5 +728,5 @@ if(hes.count==29) R.cc<-0
         mresid = mresid, cresid = cresid, y = Y, yhat = yhat, 
         stu.cov = NA, num.obs = Ny, num.student = nstudent, num.year = nyear, 
         num.teach = nteacher, yhat.m = yhat.m, sresid = sresid, 
-        yhat.s = yhat.s,iter=it)
+        yhat.s = yhat.s,iter=it,persistence=control$persistence)
 }
